#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "psemu.h"
#include "type.h"
#include "gpu.h"
#include "win32def.h"
#include "locals.h"

UINT8   *ram;

static int     initialized=0;

HMODULE GpuDLL;

long (*GPUinit)();
long (*GPUshutdown)();
long (*GPUopen)();
long (*GPUclose)();

long (*GPUreadStatus)();
long (*GPUwriteStatus)();
long (*GPUreadData)();
long (*GPUwriteData)();

long (*GPUupdateLace)();
long (*GPUdmaChain)();

long (*GPUconfigure)();
long (*GPUtest)();
long (*GPUabout)();

long (*PSEgetLibName)();
long (*PSEgetLibType)();
long (*PSEgetLibVersion)();

long (*GPUmakeSnapshot)();

void (*Flush)();

void GPU_Update(void)
{
    GPUupdateLace();
}

UINT32 GP1_Read(void)
{
    return GPUreadStatus();
}

UINT32 GP0_Read(void)
{
    return GPUreadData();
}

void GP0_Write(UINT32 data)
{
    GPUwriteData(data);
#ifdef __GNUC__
    asm (" subl $4, %esp ");
#endif
}

void GP1_Write(UINT32 code)
{
    GPUwriteStatus(code);
#ifdef __GNUC__
    asm (" subl $4, %esp ");
#endif
}

int GPU_Open(UINT32 *par)
{
    FPSEWin32 *g = (FPSEWin32 *)par;
    char       buf[256];
    char       nf[64];

    GetActiveDir(g->HInstance);
    strcpy(buf,LocalPath);
    strcat(buf,"PLUGIN\\PSEMU\\");
    g->ReadCfg("GPUPSEMU","PluginName",nf);
    strcat(buf,nf);
    if (LoadPlugin(buf) != FPSE_OK) return FPSE_ERR;

    GPUinit();
    GPUopen(g->HWnd);
#ifdef __GNUC__
    asm (" subl $4, %esp ");
#endif

    ram = g->SystemRam;
    Flush = g->FlushRec;
    g->Flags &= ~GPU_USE_DIB_UPDATE;
    initialized = 1;

    return FPSE_OK;
}

void GPU_Close(void)
{
    if (!initialized) return;

    GPUclose();
    GPUshutdown();

    FreeLibrary(GpuDLL);

    initialized = 0;
}

void GPU_ScreenShot(char *path)
{
    GPUmakeSnapshot();
}	    

void GPU_DmaExec(UINT32 adr,UINT32 bcr,UINT32 chcr)
{
    int x,size;
    UINT32 *ram32 = (UINT32 *)(ram+(adr & 0x1FFFFF));

    size = (bcr>>16)*(bcr&0xffff);

    switch(chcr){
    case 0x01000200:
        Flush(adr,adr+size*4);
        for (x=0;x<size;x++)
            ram32[x] = GPUreadData();
        break;
    case 0x01000201:
        for (x=0;x<size;x++)
             GP0_Write(ram32[x]);
        break;
    case 0x01000401:
        GPUdmaChain(ram,adr&0x1fffff);
#ifdef __GNUC__
        asm (" subl $8, %esp ");
#endif
        break;
    case 0x00000200:
    case 0x00000201:
    case 0x00000401: // disable dma
        break;
    default: printf("gpudma unknown %08x\n",(int)chcr);
    }
}
