#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "type.h"
#include "gpu.h"
#include "Win32Def.h"
#include "resource.h"
#include "psemu.h"
#include "locals.h"

#define VERSION     0x0100  // Version must be in BCD format

typedef struct {
    char FileName[32];
} PluginType;

static unsigned int PlugNum = 0;
static PluginType   PlugList[16];
static HANDLE       hinstance = (HANDLE)-1;
static FPSEWin32   *Info;

char         LocalPath[256];

int LoadPlugin(char *name)
{
    GpuDLL = LoadLibrary(name);

    if (GpuDLL < (HMODULE)(32)) return FPSE_ERR;

    GPUinit     = (long *)GetProcAddress(GpuDLL,"GPUinit");
    GPUshutdown = (long *)GetProcAddress(GpuDLL,"GPUshutdown");
    GPUopen     = (long *)GetProcAddress(GpuDLL,"GPUopen");
    GPUclose    = (long *)GetProcAddress(GpuDLL,"GPUclose");

    GPUreadStatus  = (long *)GetProcAddress(GpuDLL,"GPUreadStatus");
    GPUwriteStatus = (long *)GetProcAddress(GpuDLL,"GPUwriteStatus");
    GPUreadData    = (long *)GetProcAddress(GpuDLL,"GPUreadData");
    GPUwriteData   = (long *)GetProcAddress(GpuDLL,"GPUwriteData");

    GPUupdateLace  = (long *)GetProcAddress(GpuDLL,"GPUupdateLace");
    GPUdmaChain    = (long *)GetProcAddress(GpuDLL,"GPUdmaChain");

    GPUconfigure = (long *)GetProcAddress(GpuDLL,"GPUconfigure");
    GPUtest      = (long *)GetProcAddress(GpuDLL,"GPUtest");
    GPUabout     = (long *)GetProcAddress(GpuDLL,"GPUabout");

    PSEgetLibName    = (long *)GetProcAddress(GpuDLL,"PSEgetLibName");
    PSEgetLibType    = (long *)GetProcAddress(GpuDLL,"PSEgetLibType");
    PSEgetLibVersion = (long *)GetProcAddress(GpuDLL,"PSEgetLibVersion");

    GPUmakeSnapshot = (long *)GetProcAddress(GpuDLL,"GPUmakeSnapshot");

    return FPSE_OK;
}

static void ScanPlugins(HWND hwnd)
{
    WIN32_FIND_DATA data;
    HANDLE          h;
    char            buf[256];
    int             x;

    strcpy(buf,LocalPath);
    strcat(buf,"PSEMU\\*.dll");

    h = FindFirstFile(buf,&data);
    if (h == INVALID_HANDLE_VALUE) return;
    for (;;)
    {
        if (!(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
        {
            strcpy(buf,LocalPath);
            strcat(buf,"PSEMU\\");
            strcat(buf,data.cFileName);
            if (LoadPlugin(buf) == FPSE_OK)
            {
                if (PSEgetLibType() == PSE_LT_GPU)
                {
                    strcpy(PlugList[PlugNum].FileName,data.cFileName);
                    x = PSEgetLibVersion();
                    sprintf(buf,"%s %d.%d",
                            (char *)PSEgetLibName(), (x>>8) & 0xFF, x & 0xFF);
                    SendMessage((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),
                                CB_ADDSTRING,0,(long)buf);
                    PlugNum++;
                }
                FreeLibrary(GpuDLL);
            }
        }
        if (FindNextFile(h,&data) == FALSE) break;
    }
    FindClose(h);
}

static FPSEWin32About info = {
    FPSE_GPU,
    VERSION & 0xFF,
    VERSION >> 8,
    0,
    "LDChen",
    "GPUPsEmu",
    "This Plugin allows to use PsemuPro Gpu plugin with FPSE."
};

void GetActiveDir(HINSTANCE hInstance)
{
    char *p;

// First get the full path of the application
    GetModuleFileName(hInstance, LocalPath, 256);

// Then trim off the program name leaving just the path to the directory (C:\DIR\DIR\)
   p = LocalPath;
   while(strchr(p,'\\')) {  // while there are more separators
      p = strchr(p,'\\');   // point to next separator
      p++;                  // point to char after last separator
   }
   *p = '\0';               // terminate string
}

BOOL WINAPI
#ifdef __GNUC__
LibMain
#else
DllMain
#endif
(HANDLE hInstance, DWORD fdwReason, PVOID fImpLoad)
{
    hinstance = hInstance;

    GetActiveDir(hInstance);

    return TRUE;
}

BOOL CALLBACK ConfigDialogProc(HWND hwnd, UINT msg, WPARAM wPar, LPARAM lPar)
{
    char buf[256];
    int  number;

    switch (msg) {
    case WM_INITDIALOG:
        ScanPlugins(hwnd);

        if (PlugNum) {
            SendMessage((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),CB_SETCURSEL,0,0);
        } else {
            EnableWindow((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),FALSE);
            EnableWindow((HWND)GetDlgItem(hwnd,IDC_CONFIGURE_BUTTON),FALSE);
            EnableWindow((HWND)GetDlgItem(hwnd,IDC_TEST_BUTTON),FALSE);
            EnableWindow((HWND)GetDlgItem(hwnd,IDC_ABOUT_BUTTON),FALSE);
        }
        return TRUE;

    case WM_COMMAND:
        switch (wPar) {
        case IDC_TEST_BUTTON:
        case IDC_ABOUT_BUTTON:
        case IDC_CONFIGURE_BUTTON:
            number = SendMessage((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),
                                 CB_GETCURSEL,0,0);
            strcpy(buf,LocalPath);
            strcat(buf,"PSEMU\\");
            strcat(buf,PlugList[number].FileName);
            LoadPlugin(buf);
            switch (wPar) {
            case IDC_TEST_BUTTON:
                number = 1;
                if (GPUtest != NULL) number = GPUtest();
                switch (number) {
                case PSE_ERR_SUCCESS:
                    sprintf(buf,"This driver reports that it should work correctly!");
                    break;
                case PSE_ERR_FATAL:
                    sprintf(buf,"This driver can't run on your hardware!");
                default:
                    sprintf(buf,"This is an old style driver and does not support TEST function");
                    break;
                }
                MessageBox(hwnd,buf,"Psemu",MB_OK | MB_ICONINFORMATION);
                break;
            case IDC_ABOUT_BUTTON:
                GPUabout();
                break;
            case IDC_CONFIGURE_BUTTON:
                GPUconfigure();
                break;
            }
            FreeLibrary(GpuDLL);
            return TRUE;
        case IDOK:
            number = SendMessage((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),
                                 CB_GETCURSEL,0,0);
            Info->WriteCfg("GPUPSEMU","PluginName",PlugList[number].FileName);
        case IDCANCEL:
            EndDialog(hwnd,0);
            return TRUE;
        }
        break;
    case WM_DESTROY:
        EndDialog(hwnd,0);
        return TRUE;
    }

    return FALSE;
}

int GPU_Configure(UINT32 *par)
{
    Info = (FPSEWin32 *)par;

    DialogBox(hinstance,MAKEINTRESOURCE(IDD_DIALOG),
              Info->HWnd,ConfigDialogProc);
    return FPSE_OK;
}

void GPU_About(UINT32 *par)
{
    memcpy( par, &info, sizeof(FPSEWin32About) );
}
